"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Collection = void 0;
const promises_1 = require("node:dns/promises");
const mongodb_1 = require("./mongodb");
const errors_1 = require("./errors");
const locker_1 = require("./locker");
class Collection extends mongodb_1.MongoCollection {
    locker = new locker_1.Locker();
    constructor(db, name, options) {
        super(db, name, options);
    }
    countDocuments(filter, options) {
        return super.countDocuments(filter, options);
    }
    upsert(filter, update, options) {
        return super.updateOne(filter, update, {
            upsert: true,
            ...options
        });
    }
    withDocumentLock(filter, handle) {
        return this.lock(filter)
            .then(() => Promise.resolve(handle()))
            .then(result => this.unlockWithResult(filter, result))
            .catch(async (error) => {
            if (!(error instanceof errors_1.DocumentPinnedError)) {
                await this.unlock(filter);
            }
            throw error;
        });
    }
    async lock(filter) {
        this.locker.set(filter, {
            inTransit: true
        });
        const filters = {
            ...filter,
            _lockPinned: { $exists: false }
        };
        const isLockPinned = await super.findOne({
            ...filter,
            _lockPinned: true
        });
        const lock = {
            _lockPinned: true,
            _lockOwnerHostname: process.env.HOSTNAME || 'localhost'
        };
        const update = {
            $set: lock,
            $setOnInsert: filter
        };
        if (isLockPinned) {
            const hostname = isLockPinned._lockOwnerHostname;
            if (this.locker.has(filter)) {
                throw new errors_1.DocumentPinnedError('Document is internally locked within same Process');
            }
            const isOwnerStillAlive = await (0, promises_1.resolve)(hostname)
                .then(() => true)
                .catch(() => false);
            if (isOwnerStillAlive) {
                throw new errors_1.DocumentPinnedError('Owner is alive');
            }
        }
        return super.updateOne(filters, update, {
            upsert: true
        })
            .catch(error => {
            if (error.code === 11000) {
                throw new errors_1.DocumentPinnedError('Document failed to insert a pin');
            }
            throw error;
        });
    }
    unlock(filter) {
        this.locker.delete(filter);
        const filters = {
            ...filter,
            _lockPinned: true
        };
        const lock = {
            _lockPinned: true,
            _lockOwnerHostname: true
        };
        return super.updateOne(filters, {
            $unset: lock
        });
    }
    unlockWithResult(filter, result) {
        return this.unlock(filter).then(() => result);
    }
}
exports.Collection = Collection;
//# sourceMappingURL=collection.js.map